/** @file 
 * @brief       ""  28147-89.      
 * 
 * @copyright InfoTeCS. All rights reserved.
 */

#include <time.h>
#include <memory.h>
#include <malloc.h>
#include <stdio.h>
#include "../Gost/version.h"
#include "../Gost/28147_14.h"
#include "../Gost/block_chipher.h"
#include "../Gost/28147_89.h"

/** @brief   - */
unsigned char kMasterKeyData[32] = 
{
     0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66,0x77,
     0xfe, 0xdc, 0xba, 0x98, 0x76, 0x54, 0x32, 0x10, 0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef
};

/** @brief   -   28147-89 */
unsigned char kMasterKeyGost89[] = 
{
     0xff, 0xee, 0xdd, 0xcc, 0xbb, 0xaa, 0x99, 0x88, 0x77, 0x66, 0x55, 0x44, 0x33, 0x22, 0x11, 0x00,
     0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff
};

/**    */
#define buffSize 1 * 1024 * 1024
/**     */
#define MULTIPLIER 1000

/** @brief         1    "" */
unsigned long Gost14TimeTest(unsigned long *count)
{
     unsigned char *input_buffer;
     unsigned char *output_buffer;
     unsigned char ctx[kEcb14ContextLen];
     volatile clock_t start, end;
     unsigned long i;
     
     input_buffer = (unsigned char *)malloc(buffSize * 2);
     output_buffer = input_buffer + buffSize;

     if( !input_buffer )
          return 0;

     if(init_ecb_14(kMasterKeyData, ctx, 0, 0))
     {
          free( input_buffer );
          return 0;
     }

     memset( input_buffer, 0xDD, buffSize );
     *count = 0;
     start = clock();

     /*     .    . */
     while(clock() < start + CLOCKS_PER_SEC)
     {
          if(encrypt_ecb(ctx, input_buffer, output_buffer, buffSize))
          {
               free( input_buffer );
               return 0;
          }
          ++*count;
     }

     /*   */
     start = clock();
     for(i = 0 ; i < *count; ++i)
     {
          if(encrypt_ecb(ctx, input_buffer, output_buffer, buffSize))
          {
               free( input_buffer );
               return 0;
          }
     }
     end = clock();
     free( input_buffer );
     return *count * CLOCKS_PER_SEC * MULTIPLIER / (end - start);
}

/** @brief         1    "28147-89" */
unsigned long Gost89TimeTest(unsigned long *count)
{
     unsigned char *input_buffer;
     unsigned char *output_buffer;
     unsigned char ctx[kEcb89ContextLen];
     volatile clock_t start, end;
     unsigned int i;
     
     input_buffer = (unsigned char *)malloc(buffSize * 2);
     output_buffer = input_buffer + buffSize;
     if( !input_buffer )
          return 0;

     if(init_ecb_89(kMasterKeyGost89, ctx, 0, 0))
     {
          free( input_buffer );
          return 0;
     }
    
     memset( input_buffer, 0xDD, buffSize );
     *count = 0;

     /*     .    . */
     start = clock();
     while(clock() < start + CLOCKS_PER_SEC)
     {
          if(encrypt_ecb(ctx, input_buffer, output_buffer, buffSize))
          {
               free( input_buffer );
               return 0;
          }
          ++*count;
     }

     /*   */
     start = clock();
     for(i = 0 ; i < *count; ++i)
     {
          if(encrypt_ecb(ctx, input_buffer, output_buffer, buffSize))
          {
               free( input_buffer );
               return 0;
          }
     }
     end = clock();

     free( input_buffer );

     return *count * CLOCKS_PER_SEC * MULTIPLIER / (end - start);
}

/** @brief   */
int main()
{
     unsigned long count14;
     unsigned long time14 = Gost14TimeTest(&count14);
     unsigned long count89;
     unsigned long time89 = Gost89TimeTest(&count89);

     printf("GostTimeTest.exe started.\nversion %x.%x\n", VERSION_MAJOR, VERSION_MINOR);
     printf("Gost 28147-14 %lu.%03lu mb/s. Cycles=%lu \n", time14 / MULTIPLIER, time14 % MULTIPLIER, count14 );
     printf("Gost 28147-89 %lu.%03lu mb/s. Cycles=%lu \n", time89 / MULTIPLIER, time89 % MULTIPLIER, count89 );
     return 0;
}
